// Program: VIEWWIN.CPP
// Author: B.Zink	- CIS ID 70224,37
// (c) Copyright 1992 - All Rights Reserved
// Version: 1.00
// ---------------------------------------------------------------
// Revisions:
// ---------------------------------------------------------------
// August 27, 1992 - No revisions
// ---------------------------------------------------------------
//
// Remarks:

#include <owl.h>
#include <bwcc.h>
#include <windows.h>
#include <listbox.h>
#include <combobox.h>
#include <stdio.h>
#include <ctype.h>

#include "ids.h"
#include "viewwin.hpp"

extern Array LinesInFile;
extern int numberLines;
const int MAXLINES = 4000;
extern POINT defaultSize;
extern PString thisString;

TViewWindow::TViewWindow( PTWindowsObject AParent, char *AFile )
				: TWindow( AParent, AFile )
{
	 // Add horizontal and vertical scrollbars
	 Attr.Style = WS_VSCROLL | WS_HSCROLL | WS_VISIBLE | WS_MAXIMIZEBOX | WS_THICKFRAME;

	 // Dummy values for the scrolling window
	 Scroller = new TScroller( this, 10, 10, 100, 100 );

	 // Allocate memory for filename;   	
	 filename = new char[80];
	 _fstrcpy(filename, AFile);
}

// This function is extremely specific to the friend I wrote this for.
// but can be useful.
BOOL ValidateLine( char *thisLine, char *lastLine );

void TViewWindow::SetupWindow()
{
	FILE *fp;
	char buffer[200],lastLine[200];

	// use stdio stuff to open a file in TEXT mode
   // NOTE: Don't care to do Binary files this is TEXT stuff
	if ( (fp = fopen(filename,"rt")) == NULL) {
   	wsprintf(buffer,"Can't open file: %s",filename);
		BWCCMessageBox(NULL,buffer,"File Error",MB_OK);
		return;
   }

	BOOL ok = TRUE;  // These two variables are simply a check to eliminate
	int index = 0;   //   the possibility of an infinite loop while reading

	lastLine[0] = 0;
	char errorMsg[80];
   errorMsg[0] = 0;
	while (fgets(buffer,199,fp)!=NULL && ok) {
		char *ptr = _fstrchr( buffer,'\n'); // remove the \n from the line
		if (ptr)
			*ptr = 0;
		else {
			wsprintf(errorMsg,"Data Error in File: %s.\nLine feed (\\n) not found",
         						filename);
			BWCCMessageBox(NULL, errorMsg,"File Data Error", MB_OK | MB_ICONSTOP);
         break;
		}
		// Can modify the input buffer with the ValidateLine function
		// I had specific purpose, i.e. to remove a single character
		// and will add tab fills later. 
		if (ValidateLine(buffer,lastLine)) {  
			ptr = buffer;
         // SPECIFIC TO USER - WORTHLESS TO NORMAL HUMANS 
			if ( toupper(filename[5]) == 'L')
				ptr++;

			PString thisString = new String(ptr);
			// Check for memory errors when allocating memory
         // could be cleaner error checking, i.e. what does break do ???
			if( thisString == NULL) {
				wsprintf(errorMsg,"Out of Memory After:\n%d Lines Read",index);
				BWCCMessageBox(NULL,errorMsg,"Memory Error",MB_ICONSTOP | MB_OK);
				break;
			}
         // Add the string to the array
			LinesInFile.add( *((PString)(thisString)) );
			// increment the number of lines added to the array
			// really don't have to do this, as GetItemsInContainer() would work
         // but easy way to check for infinite loop
			index++;
      }
		if (index == MAXLINES)	// check for a maximum line condition
			ok = FALSE;
      _fstrcpy(lastLine,buffer);
   }
	fclose(fp);					// close the file

	// extremely LIMP error recovery technique
	// (hey, I wrote this in an afternoon)
	if (errorMsg[0]) {
      char msg[3][80];
		char *ptr[3];
		ptr[0] = strtok( errorMsg, "\n");
		if(ptr[0]) {
      	_fstrcpy(msg[0],ptr[0]);
			ptr[1] = strtok(NULL,"\n");
      }
		if(ptr[1]) {
      	_fstrcpy(msg[1],ptr[1]);
			ptr[2] = strtok(NULL,"\n");
		}
      msg[2][0] = 0;
		LinesInFile.ownsElements(TShouldDelete::Delete);
		LinesInFile.flush(TShouldDelete::Delete);
		index=0;
		// make window say same thing as Message Box
		while ( msg[index][0] ) {
			thisString = new String(msg[index]); 
			LinesInFile.add( *((PString)(thisString)) );
			index++;
		}
   }
		
	numberLines = index;		// set global variable = to number lines read
   									// this keeps track of the scrolling operation
	TWindow::SetupWindow();   // Don't forget to setup the window

	memset(&lFont,0,sizeof(LOGFONT));

	// Use the terminal font i.e. font used for MS-DOS window
   // Since usually displaying a DOS file - FIXED PITCH
	lFont.lfHeight          = defaultSize.y;
   lFont.lfWidth				= defaultSize.x;
	lFont.lfCharSet			= OEM_CHARSET;
	_fstrcpy(lFont.lfFaceName,"Terminal");

	// Create the new font with given parameters
	// Use LOGFONT for easy change of size

	windowFont = CreateFontIndirect(&lFont);

	HDC hDC = GetDC( HWindow );
   if ( hDC ) {
		TEXTMETRIC tm;
      SelectObject(hDC, windowFont ); // Select Font into DC to get proper
		GetTextMetrics( hDC, &tm );     // sizes for scrolling
      ReleaseDC( HWindow, hDC );
      Scroller->XUnit = tm.tmAveCharWidth;
		Scroller->YUnit = tm.tmHeight + tm.tmExternalLeading + tm.tmOverhang;
		Scroller->SetRange( 1500, numberLines);  // set scroll ranges - 1500 is bogus
      													  // numberLines is from file	
	}
	else {
		BWCCMessageBox(NULL,"GDI - HDC unknown error!\n"
							 "TViewWindow::SetupWindow",
							 "Fatal - Call programming Dude!",
					  		  MB_OK | MB_ICONSTOP);
		return;
   }
}

BOOL ValidateLine( char *thisLine, char *lastLine)
{
// SPECIFIC USER STUFF - ADD YOUR OWN VALIDATION STUFF WHEN READING

//	if (thisLine[0] == '1')
//		return FALSE;
	if (_fstrncmp(" LINE",thisLine,5) == 0)
		return FALSE;
	if (_fstrncmp(" LINE",lastLine,5) ==0 && _fstrncmp(thisLine," ---",4)==0)
      return FALSE;
	return TRUE;
}

// Trap the Non-Client Message to keep from letting user move this window
// with the title bar - User can Maximize window, but not move it

void TViewWindow::WMNCLButtonDown(RTMessage Msg )
{
	if (Msg.WParam == HTCAPTION)
		MessageBeep(0);
	else
		DefWndProc( Msg );
}

TViewWindow::~TViewWindow()
{
	if (filename)
		delete filename;
	DeleteObject( windowFont );

	// Dont flush the stuff here as print routine needs the info

	//	LinesInFile.ownsElements(TShouldDelete::Delete);
	//	LinesInFile.flush(TShouldDelete::Delete);
}

void TViewWindow::Paint( HDC PaintDC, PAINTSTRUCT& )
{
	UINT  index;
	SelectObject( PaintDC, windowFont);
	Scroller->SetPageSize();

	char *buffer;
	buffer = new char[200];

	// An amazing fact - Even though it looks like all lines will be printed,
	// they won't be. For your own beliefs uncomment the following code:

	// static BOOL blue = TRUE;
	// blue = !blue;
	// SetTextColor(PaintDC,(blue)?RGB(0,0,255):RGB(0,0,0));
   	
	for( index=Scroller->YPos; index<=Scroller->YPos+Scroller->YPage+1; index++ ){
   	// get the lines out of the array
		if ( index < numberLines )  {
			RObject AnObject = LinesInFile[index];
			if (AnObject != NOOBJECT) {
				String AString = *((PString)(&AnObject));
            _fstrcpy(buffer, AString);
				TextOut( PaintDC, 2, index * Scroller->YUnit, buffer, lstrlen( buffer ) );
			}
      }
	}

	delete buffer;
}

TViewDialog::TViewDialog(PTWindowsObject AParent, LPSTR ATitle, char *AFile)
				:TDialog( AParent, ATitle)
{
	fileWindow = new TViewWindow( this, AFile );
	sizeComboBox = new TComboBox( this, ID_CBFONTSIZE, 8);
}

TViewDialog::~TViewDialog()
{
	if (fileWindow)
		delete fileWindow;
   delete sizeComboBox;
}

void TViewDialog::SetupWindow()
{
	TDialog::SetupWindow();

	char lastSize[5];

	// could have set this up in constructor, but didn't
	MoveWindow( fileWindow->HWindow, 28,97,580,300,TRUE);
	// Show the window - the window is NOT a child because I wanted it to
   // Fill the entire screen with maximize box, so must "Show" it
	fileWindow->Show(SW_SHOW);

   // again, weak attempt at error checking - there are better ways.
	if (!IsWindowVisible(fileWindow->HWindow)) {
   	ShutDownWindow();
		return;
   }
   // Make the combobox have a fixed font
	HFONT fixed = GetStockObject( SYSTEM_FIXED_FONT);
	SendMessage( sizeComboBox->HWindow,WM_SETFONT, fixed, TRUE);

   // Fill the ComboBox with available font sizes
	FillFontSizes();
	// Update current font size in combobox
	char currentFont[8];
	wsprintf(currentFont,"%2d x %2d",defaultSize.x,defaultSize.y);
	sizeComboBox->SetSelString( (LPSTR)currentFont,-1);
}

void TViewDialog::WMHelp( RTMessage )
{
	// Nice try !
	MessageBeep(0);
}

void TViewDialog::WMChangeSize( RTMessage Msg )
{
	// available sizes -  " 4 x  6", " 5 x 12", " 6 x  8", " 7 x 12", " 8 x  8"
	//
	static POINT sizes[] = { {4,6}, {5,12}, {6,8}, {7,12}, {8,8}, {0,0} };

	if (Msg.LP.Hi == CBN_SELCHANGE) {
		char fSize[8];
		sizeComboBox->GetText( fSize, 8);
		int  fontSize = atoi(fSize) - 4;
      // Set up LOGFONT information from combobox
		fileWindow->lFont.lfWidth  = defaultSize.x = sizes[fontSize].x;
		fileWindow->lFont.lfHeight = defaultSize.y = sizes[fontSize].y;
		// Delete the old window font 
		DeleteObject( fileWindow->windowFont );
      // Create a new font
		fileWindow->windowFont = CreateFontIndirect( &(fileWindow->lFont));
      // Get information for proper scrolling
		TEXTMETRIC tm;
		HDC theDC = GetDC( fileWindow->HWindow);
      SelectObject( theDC, fileWindow->windowFont);
		GetTextMetrics( theDC, &tm);
		ReleaseDC( fileWindow->HWindow, theDC);
		fileWindow->Scroller->XUnit = tm.tmAveCharWidth;
		fileWindow->Scroller->YUnit = tm.tmHeight + tm.tmExternalLeading + tm.tmOverhang;

      // repaint the file window with the new font
		InvalidateRect( fileWindow->HWindow, NULL, TRUE);
	}
}

void TViewDialog::FillFontSizes()
{
	static char *sizes[] = { " 4 x  6", " 5 x 12", " 6 x  8", " 7 x 12", " 8 x  8", "\0"};
	char lastSize[5];

   for (int i=0;sizes[i][0];i++)
		sizeComboBox->AddString( sizes[i] );
}
